@@ -3,60 +3,118 @@ package Moo;
 use strictures 1;
 use Moo::_Utils;
 use B 'perlstring';
+use Sub::Defer ();
 
-our $VERSION = '0.009013'; # 0.9.13
+our $VERSION = '0.091011'; # 0.91.11
 $VERSION = eval $VERSION;
 
+require Moo::sification;
+
 our %MAKERS;
 
+sub _install_tracked {
+  my ($target, $name, $code) = @_;
+  $MAKERS{$target}{exports}{$name} = $code;
+  _install_coderef "${target}::${name}" => "Moo::${name}" => $code;
+}
+
 sub import {
   my $target = caller;
   my $class = shift;
   strictures->import;
   return if $MAKERS{$target}; # already exported into this package
-  *{_getglob("${target}::extends")} = sub {
+  $MAKERS{$target} = {};
+  _install_tracked $target => extends => sub {
     _load_module($_) for @_;
     # Can't do *{...} = \@_ or 5.10.0's mro.pm stops seeing @ISA
     @{*{_getglob("${target}::ISA")}{ARRAY}} = @_;
+    if (my $old = delete $Moo::MAKERS{$target}{constructor}) {
+      delete _getstash($target)->{new};
+      Moo->_constructor_maker_for($target)
+         ->register_attribute_specs(%{$old->all_attribute_specs});
+    }
+    no warnings 'once'; # piss off. -- mst
+    $Moo::HandleMoose::MOUSE{$target} = [
+      grep defined, map Mouse::Util::find_meta($_), @_
+    ] if $INC{"Mouse.pm"};
+    $class->_maybe_reset_handlemoose($target);
+    return;
   };
-  *{_getglob("${target}::with")} = sub {
-    { local $@; require Moo::Role; }
-    die "Only one role supported at a time by with" if @_ > 1;
-    Moo::Role->apply_role_to_package($target, $_[0]);
+  _install_tracked $target => with => sub {
+    require Moo::Role;
+    Moo::Role->apply_roles_to_package($target, @_);
+    $class->_maybe_reset_handlemoose($target);
   };
-  $MAKERS{$target} = {};
-  *{_getglob("${target}::has")} = sub {
+  _install_tracked $target => has => sub {
     my ($name, %spec) = @_;
-    ($MAKERS{$target}{accessor} ||= do {
-      { local $@; require Method::Generate::Accessor; }
-      Method::Generate::Accessor->new
-    })->generate_method($target, $name, \%spec);
     $class->_constructor_maker_for($target)
           ->register_attribute_specs($name, \%spec);
+    $class->_accessor_maker_for($target)
+          ->generate_method($target, $name, \%spec);
+    $class->_maybe_reset_handlemoose($target);
+    return;
   };
   foreach my $type (qw(before after around)) {
-    *{_getglob "${target}::${type}"} = sub {
-      { local $@; require Class::Method::Modifiers; }
+    _install_tracked $target => $type => sub {
+      require Class::Method::Modifiers;
       _install_modifier($target, $type, @_);
+      return;
     };
   }
   {
     no strict 'refs';
     @{"${target}::ISA"} = do {
-      { local $@; require Moo::Object; } ('Moo::Object');
+      require Moo::Object; ('Moo::Object');
     } unless @{"${target}::ISA"};
   }
+  if ($INC{'Moo/HandleMoose.pm'}) {
+    Moo::HandleMoose::inject_fake_metaclass_for($target);
+  }
+}
+
+sub unimport {
+  my $target = caller;
+  _unimport_coderefs($target, $MAKERS{$target});
+}
+
+sub _maybe_reset_handlemoose {
+  my ($class, $target) = @_;
+  if ($INC{"Moo/HandleMoose.pm"}) {
+    Moo::HandleMoose::maybe_reinject_fake_metaclass_for($target);
+  }
+}
+
+sub _accessor_maker_for {
+  my ($class, $target) = @_;
+  return unless $MAKERS{$target};
+  $MAKERS{$target}{accessor} ||= do {
+    my $maker_class = do {
+      if (my $m = do {
+            if (my $defer_target = 
+                  (Sub::Defer::defer_info($target->can('new'))||[])->[0]
+              ) {
+              my ($pkg) = ($defer_target =~ /^(.*)::[^:]+$/);
+              $MAKERS{$pkg} && $MAKERS{$pkg}{accessor};
+            } else {
+              undef;
+            }
+          }) {
+        ref($m);
+      } else {
+        require Method::Generate::Accessor;
+        'Method::Generate::Accessor'
+      }
+    };
+    $maker_class->new;
+  }
 }
 
 sub _constructor_maker_for {
   my ($class, $target, $select_super) = @_;
   return unless $MAKERS{$target};
   $MAKERS{$target}{constructor} ||= do {
-    {
-      local $@;
-      require Method::Generate::Constructor;
-      require Sub::Defer;
-    }
+    require Method::Generate::Constructor;
+    require Sub::Defer;
     my ($moo_constructor, $con);
 
     if ($select_super && $MAKERS{$select_super}) {
@@ -78,20 +136,22 @@ sub _constructor_maker_for {
         $moo_constructor = 1; # no other constructor, make a Moo one
       }
     };
-    Method::Generate::Constructor
+    ($con ? ref($con) : 'Method::Generate::Constructor')
       ->new(
         package => $target,
-        accessor_generator => do {
-          { local $@; require Method::Generate::Accessor; }
-          Method::Generate::Accessor->new;
-        },
+        accessor_generator => $class->_accessor_maker_for($target),
         construction_string => (
           $moo_constructor
             ? ($con ? $con->construction_string : undef)
             : ('$class->'.$target.'::SUPER::new(@_)')
         ),
-        subconstructor_generator => (
-          $class.'->_constructor_maker_for($class,'.perlstring($target).')'
+        subconstructor_handler => (
+          '      if ($Moo::MAKERS{$class}) {'."\n"
+          .'        '.$class.'->_constructor_maker_for($class,'.perlstring($target).');'."\n"
+          .'        return $class->new(@_)'.";\n"
+          .'      } elsif ($INC{"Moose.pm"} and my $meta = Class::MOP::get_metaclass_by_name($class)) {'."\n"
+          .'        return $meta->new_object(@_);'."\n"
+          .'      }'."\n"
         ),
       )
       ->install_delayed
@@ -182,6 +242,35 @@ Hence - Moo exists as its name - Minimal Object Orientation - with a pledge
 to make it smooth to upgrade to L<Moose> when you need more than minimal
 features.
 
+=head1 Moo and Moose - NEW, EXPERIMENTAL
+
+If L<Moo> detects L<Moose> being loaded, it will automatically register
+metaclasses for your L<Moo> and L<Moo::Role> packages, so you should be able
+to use them in L<Moose> code without it ever realising you aren't using
+L<Moose> everywhere.
+
+Extending a L<Moose> class or consuming a L<Moose::Role> should also work.
+
+So should extending a L<Mouse> class or consuming a L<Mouse::Role>.
+
+This means that there is no need for anything like L<Any::Moose> for Moo
+code - Moo and Moose code should simply interoperate without problem. To
+handle L<Mouse> code, you'll likely need an empty Moo role or class consuming
+or extending the L<Mouse> stuff since it doesn't register true L<Moose>
+metaclasses like we do.
+
+However, these features are new as of 0.91.0 (0.091000) so while serviceable,
+they are absolutely certain to not be 100% yet; please do report bugs.
+
+If you need to disable the metaclass creation, add:
+
+  no Moo::sification;
+
+to your code before Moose is loaded, but bear in mind that this switch is
+currently global and turns the mechanism off entirely, so don't put this
+in library code, only in a top level script as a temporary measure while
+you send a bug report.
+
 =head1 IMPORTED METHODS
 
 =head2 new
@@ -194,13 +283,12 @@ or
 
 =head2 BUILDARGS
 
- around BUILDARGS => sub {
-   my $orig = shift;
+ sub BUILDARGS {
    my ( $class, @args ) = @_;
 
    unshift @args, "attr1" if @args % 2 == 1;
 
-   return $class->$orig(@args);
+   return { @args };
  };
 
  Foo::Bar->new( 3 );
@@ -255,10 +343,13 @@ them like 'use base' would.
 =head2 with
 
  with 'Some::Role1';
- with 'Some::Role2';
 
-Composes a L<Role::Tiny> into current class.  Only one role may be composed in
-at a time to allow the code to remain as simple as possible.
+or
+
+ with 'Some::Role1', 'Some::Role2';
+
+Composes one or more L<Moo::Role> (or L<Role::Tiny>) roles into the current
+class.  An error will be raised if these roles have conflicting methods.
 
 =head2 has
 
@@ -274,9 +365,24 @@ The options for C<has> are as follows:
 
 =item * is
 
-B<required>, must be C<ro> or C<rw>.  Unsurprisingly, C<ro> generates an
-accessor that will not respond to arguments; to be clear: a getter only. C<rw>
-will create a perlish getter/setter.
+B<required>, may be C<ro>, C<lazy>, C<rwp> or C<rw>.
+
+C<ro> generates an accessor that dies if you attempt to write to it - i.e.
+a getter only - by defaulting C<reader> to the name of the attribute.
+
+C<lazy> generates a reader like C<ro>, but also sets C<lazy> to 1 and
+C<builder> to C<_build_${attribute_name}> to allow on-demand generated
+attributes.  This feature was my attempt to fix my incompetence when
+originally designing C<lazy_build>, and is also implemented by
+L<MooseX::AttributeShortcuts>.
+
+C<rwp> generates a reader like C<ro>, but also sets C<writer> to
+C<_set_${attribute_name}> for attributes that are designed to be written
+from inside of the class, but read-only from outside.
+This feature comes from L<MooseX::AttributeShortcuts>.
+
+C<rw> generates a normal getter/setter by defaulting C<accessor> to the
+name of the attribute.
 
 =item * isa
 
@@ -290,6 +396,28 @@ one should do
 
 L<Sub::Quote aware|/SUB QUOTE AWARE>
 
+Since L<Moo> does B<not> run the C<isa> check before C<coerce> if a coercion
+subroutine has been supplied, C<isa> checks are not structural to your code
+and can, if desired, be omitted on non-debug builds (although if this results
+in an uncaught bug causing your program to break, the L<Moo> authors guarantee
+nothing except that you get to keep both halves).
+
+If you want L<MooseX::Types> style named types, look at
+L<MooX::Types::MooseLike>.
+
+To cause your C<isa> entries to be automatically mapped to named
+L<Moose::Meta::TypeConstraint> objects (rather than the default behaviour
+of creating an anonymous type), set:
+
+  $Moo::HandleMoose::TYPE_MAP{$isa_coderef} = sub {
+    require MooseX::Types::Something;
+    return MooseX::Types::Something::TypeName();
+  };
+
+Note that this example is purely illustrative; anything that returns a
+L<Moose::Meta::TypeConstraint> object or something similar enough to it to
+make L<Moose> happy is fine.
+
 =item * coerce
 
 Takes a coderef which is meant to coerce the attribute.  The basic idea is to
@@ -299,7 +427,10 @@ do something like the following:
    $_[0] + 1 unless $_[0] % 2
  },
 
-Coerce does not require C<isa> to be defined.
+Note that L<Moo> will always fire your coercion - this is to permit
+isa entries to be used purely for bug trapping, whereas coercions are
+always structural to your code. We do, however, apply any supplied C<isa>
+check after the coercion has run to ensure that it returned a valid value.
 
 L<Sub::Quote aware|/SUB QUOTE AWARE>
 
@@ -324,8 +455,13 @@ Takes a hashref
 
 =item * trigger
 
-Takes a coderef which will get called any time the attribute is set. Coderef
-will be invoked against the object with the new value as an argument.
+Takes a coderef which will get called any time the attribute is set. This
+includes the constructor. Coderef will be invoked against the object with the
+new value as an argument.
+
+If you set this to just C<1>, it generates a trigger which calls the
+C<_trigger_${attr_name}> method on C<$self>. This feature comes from
+L<MooseX::AttributeShortcuts>.
 
 Note that Moose also passes the old value, if any; this feature is not yet
 supported.
@@ -349,8 +485,10 @@ L<Sub::Quote aware|/SUB QUOTE AWARE>
 
 Takes a method name which will return true if an attribute has a value.
 
-A common example of this would be to call it C<has_$foo>, implying that the
-object has a C<$foo> set.
+If you set this to just C<1>, the predicate is automatically named
+C<has_${attr_name}> if your attribute's name does not start with an
+underscore, or <_has_${attr_name_without_the_underscore}> if it does.
+This feature comes from L<MooseX::AttributeShortcuts>.
 
 =item * builder
 
@@ -363,10 +501,18 @@ Moo will call
 
   $self->$builder;
 
+If you set this to just C<1>, the predicate is automatically named
+C<_build_${attr_name}>.  This feature comes from L<MooseX::AttributeShortcuts>.
+
 =item * clearer
 
 Takes a method name which will clear the attribute.
 
+If you set this to just C<1>, the clearer is automatically named
+C<clear_${attr_name}> if your attribute's name does not start with an
+underscore, or <_clear_${attr_name_without_the_underscore}> if it does.
+This feature comes from L<MooseX::AttributeShortcuts>.
+
 =item * lazy
 
 B<Boolean>.  Set this if you want values for the attribute to be grabbed
@@ -400,6 +546,7 @@ leaks.
 Takes the name of the key to look for at instantiation time of the object.  A
 common use of this is to make an underscored attribute have a non-underscored
 initialization name. C<undef> means that passing the value in on instantiation
+is ignored.
 
 =back
 
@@ -432,33 +579,64 @@ aware can take advantage of this.
 
 =head1 INCOMPATIBILITIES WITH MOOSE
 
-You can only compose one role at a time.  If your application is large or
-complex enough to warrant complex composition, you wanted L<Moose>.
-
-There is no complex type system.  C<isa> is verified with a coderef, if you
+There is no built in type system.  C<isa> is verified with a coderef, if you
 need complex types, just make a library of coderefs, or better yet, functions
-that return quoted subs.
+that return quoted subs. L<MooX::Types::MooseLike> provides a similar API
+to L<MooseX::Types::Moose> so that you can write
+
+  has days_to_live => (is => 'ro', isa => Int);
+
+and have it work with both; it is hoped that providing only subrefs as an
+API will encourage the use of other type systems as well, since it's
+probably the weakest part of Moose design-wise.
 
 C<initializer> is not supported in core since the author considers it to be a
-bad idea but may be supported by an extension in future.
+bad idea but may be supported by an extension in future. Meanwhile C<trigger> or
+C<coerce> are more likely to be able to fulfill your needs.
 
 There is no meta object.  If you need this level of complexity you wanted
 L<Moose> - Moo succeeds at being small because it explicitly does not
-provide a metaprotocol.
+provide a metaprotocol. However, if you load L<Moose>, then
 
-No support for C<super>, C<override>, C<inner>, or C<augment> - override can
-be handled by around albeit with a little more typing, and the author considers
-augment to be a bad idea.
+  Class::MOP::class_of($moo_class_or_role)
+
+will return an appropriate metaclass pre-populated by L<Moo>.
+
+No support for C<super>, C<override>, C<inner>, or C<augment> - the author
+considers augment to be a bad idea, and override can be translated:
+
+  override foo => sub {
+    ...
+    super();
+    ...
+  };
+
+  around foo => sub {
+    my ($orig, $self) = (shift, shift);
+    ...
+    $self->$orig(@_);
+    ...
+  };
+
+The C<dump> method is not provided by default. The author suggests loading
+L<Devel::Dwarn> into C<main::> (via C<perl -MDevel::Dwarn ...> for example) and
+using C<$obj-E<gt>$::Dwarn()> instead.
 
 L</default> only supports coderefs, because doing otherwise is usually a
 mistake anyway.
 
-C<lazy_build> is not supported per se, but of course it will work if you
-manually set all the options it implies.
+C<lazy_build> is not supported; you are instead encouraged to use the
+C<is => 'lazy'> option supported by L<Moo> and L<MooseX::AttributeShortcuts>.
 
 C<auto_deref> is not supported since the author considers it a bad idea.
 
-C<documentation> is not supported since it's a very poor replacement for POD.
+C<documentation> will show up in a L<Moose> metaclass created from your class
+but is otherwise ignored. Then again, L<Moose> ignores it as well, so this
+is arguably not an incompatibility.
+
+Since C<coerce> does not require C<isa> to be defined but L<Moose> does
+require it, the metaclass inflation for coerce-alone is a trifle insane
+and if you attempt to subtype the result will almost certainly break.
 
 Handling of warnings: when you C<use Moo> we enable FATAL warnings.  The nearest
 similar invocation for L<Moose> would be:
@@ -468,7 +646,7 @@ similar invocation for L<Moose> would be:
 
 Additionally, L<Moo> supports a set of attribute option shortcuts intended to
 reduce common boilerplate.  The set of shortcuts is the same as in the L<Moose>
-module L<MooseX::AttributeShortcuts>.  So if you:
+module L<MooseX::AttributeShortcuts> as of its version 0.009+.  So if you:
 
     package MyClass;
     use Moo;
@@ -481,6 +659,29 @@ The nearest L<Moose> invocation would be:
     use warnings FATAL => "all";
     use MooseX::AttributeShortcuts;
 
+or, if you're inheriting from a non-Moose class,
+
+    package MyClass;
+
+    use Moose;
+    use MooseX::NonMoose;
+    use warnings FATAL => "all";
+    use MooseX::AttributeShortcuts;
+
+Finally, Moose requires you to call
+
+    __PACKAGE__->meta->make_immutable;
+
+at the end of your class to get an inlined (i.e. not horribly slow)
+constructor. Moo does it automatically the first time ->new is called
+on your class.
+
+=head1 SUPPORT
+
+Users' IRC: #moose on irc.perl.org
+
+Development and contribution IRC: #web-simple on irc.perl.org
+
 =head1 AUTHOR
 
 mst - Matt S. Trout (cpan:MSTROUT) <mst@shadowcat.co.uk>
@@ -503,6 +704,8 @@ ajgb - Alex J. G. Burzyński (cpan:AJGB) <ajgb@cpan.org>
 
 doy - Jesse Luehrs (cpan:DOY) <doy at tozt dot net>
 
+perigrin - Chris Prather (cpan:PERIGRIN) <chris@prather.org>
+
 =head1 COPYRIGHT
 
 Copyright (c) 2010-2011 the Moo L</AUTHOR> and L</CONTRIBUTORS>
